#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Créé Mars 2024
@auteur: David ALBERTO (www.astrolabe-science.fr)

QUELQUES FONCTIONS UTILES EN GNOMONIQUE.
"""
from math import sin, cos, tan, asin, acos, atan, radians, degrees, pi
import datetime as dt

eps = radians(23.439)# obliquité de l'écliptique

#  -------------------------- FONCTIONS

def rang(jour, mois):
    """
    Parameters
    ----------
    jour, mois : TYPE : deux entiers
        DESCRIPTION : n° du jour dans le mois, n° du mois dans l'année
    Returns
    -------
    Renvoie le rang du jour dans l'année (entier).
    Année non bissextile.
    """
    date = dt.date(2026, mois, jour)
    unjanvier = dt.date(2026,1,1)
    J = (date - unjanvier).days + 1
    return J

def declinaison(J, deg = False):
    """
    ----------
    J : TYPE : entier
        DESCRIPTION : rang du jour dans l'année.

    Returns
    -------
    Déclinaison du Soleil (radians par défaut, degrés si deg=True).
    Cette modélisation ne donne qu'une indication.
    """
    long = (2*pi*(J-81))/365.2422
    decl = asin(sin(eps)*sin(long))
    if deg:decl = degrees(decl)
    return decl # en radians

def declinaison_2(date, deg = False):
    """
    D'après D. Savoie (Gnomonique), valable -> 2050.
    Parameters
    ----------
    date : TYPE : date au format datetime : dt.datetime(annee, mois , jour)
        DESCRIPTION : date du jour souhaité.
    Returns
    -------
    Renvoie la déclinaison du Soleil à la date demandée (radians par défaut, 
                                                         en degrés si deg=True).
    """
    Tzero = dt.datetime(2000, 1, 1, 12) #  1er janvier 2000 12hTU, date de référence
    T = (date - Tzero).days
    T = T + (date - Tzero).seconds / 3600 / 24
    LM = (280.460 + 0.9856474 * T) % 360
    M = (357.528 + 0.9856003 * T) % 360
    long = (LM + 1.915 * sin(radians(M)) + 0.020 * sin(radians(2*M))) % 360
    # AD = arctan(cos(eps) * tan(radians(long)))
    declinaison = asin(sin(eps) * sin(radians(long)))
    if deg:
        declinaison = degrees(declinaison)
    return declinaison

def arc_semidiurne(latitude, rang, deg = False):
    """
    latitude : TYPE float
        DESCRIPTION. latitude d'un lieu, en radians.
    rang : TYPE : entier
        DESCRIPTION. rang du jour dans l'année (de 1 à 365)
    Returns
    -------
    L'angle horaire au lever/au coucher du Soleil
    (en radians par défaut, degrés si deg=True).
    """
    decl = declinaison(rang)
    H0 = acos(- tan(latitude) * tan(decl))
    if deg:
        H0 = degrees(H0)
    return H0

def azimut(latitude, J, H, deg = False):
    """
    latitude : float
    latitude du lieu (radians).
    J (entier) : rang du jour dans l'année (1 à 365)
    H (float) : angle horaire du Soleil (radians, par défaut, degrés si deg=True)
    -----------------
    Renvoie l'azimut du Soleil (radians), compté depuis le sud.
    """
    decl = declinaison(J)
    A = atan( sin(H) / ( sin(latitude) * cos(H) - cos(latitude) * tan(decl) )  )
    if sin(A)<0:
        if sin(H)>0:
            A=A+pi
    else:
       if sin(H)<0:
        A=A-pi
    if deg:
        A = degrees(A)
    return A

def hauteur(latitude, J, H, deg = False):
    """
    latitude : float
    latitude du lieu (radians).
    J (entier) : rang du jour dans l'année (1 à 365)
    H (float) : angle horaire du Soleil (radians)
    ------------------
    Renvoie la hauteur du Soleil en radians, par défaut, degrés si deg = True.
    """
    decl = declinaison(J)
    sinh = sin(latitude)*sin(decl) + cos(latitude)*cos(decl)*cos(H)
    h = asin(sinh)
    if deg:
        h = degrees(h)
    return h

def H_eclairement(J, D, latitude):
    """
    latitude (float) : latitude du lieu (radians).
    J (entier) : rang du jour dans l'année (1 à 365)
    H (float) : angle horaire du Soleil (radians)
    -------------
    Renvoie les angles horaires extrêmes d'éclairement du mur, en degres (tuple).
    """
    decl = declinaison(J)
    S = atan(tan(D) / sin(latitude))
    t = acos(cos(S) * tan(decl) * cos(latitude) / sin(latitude))
    Hmin = S - t
    Hmax = S + t
    return degrees(Hmin), degrees(Hmax)

def lever_coucher(J, latitude):
    """
    Parameters
    ----------
    J : entier
        DESCRIPTION : rang du jour dans l'année (1 à 365)
    latitude : float
        DESCRIPTION : latitude du lieu (degres).
    Returns
    -------
    Renvoie les heures de lever et de coucher du Soleil (tuple) en format décimal.

    """
    lat_rad = radians(latitude)
    H0 = arc_semidiurne(lat_rad, J, deg = True)
    Hlever = 12 - H0/15
    Hcoucher = 12 + H0/15
    return Hlever, Hcoucher

def HHmmss(H, sec=False):
    """
    Parameters
    ----------
    H : TYPE : float
        DESCRIPTION : heure en format décimal.

    -------
    Returns : heure en format string (degrés, min, seconde si option "sec")
    """
    heures = int(H)
    minutes = round((H-heures)*60)
    heure_60 = f"{heures}h{minutes}min"
    if sec:
        minutes = int((H-heures)*60)
        secondes = round( (( (H-heures) * 60)-minutes)*60 )
        heure_60 = f"{heures}h{minutes}min{secondes}s"
    return heure_60

def equation_temps(date):
    """
    D'après D. Savoie (Gnomonique), valable -> 2050.
    Parameters
    ----------
    date : TYPE : date au format datetime : dt.datetime(annee, mois , jour)
        DESCRIPTION : date du jour souhaité.
    Returns
    -------
    Renvoie la valeur de l'équation du temps à la date demandée,
    sous forme de tuple : (valeur décimale, string en format min:sec).
    """
    Tzero = dt.datetime(2000, 1, 1, 12) #  1er janvier 2000 12hTU, date de référence
    T = (date - Tzero).days
    LM = (280.460 + 0.9856474 * T) % 360
    M = (357.528 + 0.9856003 * T) % 360
    long = (LM + 1.915 * sin(radians(M)) + 0.020 * sin(radians(2*M))) % 360
    AD = atan(cos(eps) * tan(radians(long)))
    EdT = (degrees(AD) - LM) *4
    minutes = int(EdT)
    secondes = round((abs(EdT - minutes)) * 60,1)
    return EdT, f"{minutes}min{secondes}s"






































